@@ -36,8 +36,107 @@ if (is.null(getGeneric("varTransform"))) {
   setGeneric("varTransform", function(object, ...) standardGeneric("varTransform"))
 }
 
+if (is.null(getGeneric("numDetectedMSI"))) {
+  setGeneric("numDetectedMSI", function(object, ...) standardGeneric("numDetectedMSI"))
+}
+
+if (is.null(getGeneric("totalIonCountMSI"))) {
+  setGeneric("totalIonCountMSI", function(object, ...) standardGeneric("totalIonCountMSI"))
+}
+
+if (is.null(getGeneric('PCAImage'))) {
+  setGeneric("PCAImage", function(object, ...) standardGeneric("PCAImage"))
+}
+
 ## Methods ---------------------------------------------------------------------
 
+
+## PCAImage ----
+
+#' Generates an RGB msImage representing the first 3 principal components. This
+#' image can be used to qualitatively evaluate the spatial heterogeneity of the
+#' sample.
+#' 
+#' @param object \link{msi.dataset-class} object.
+#' @param alignToSample boolean (default = TRUE). If TRUE, the principal component
+#' scores are aligned to the pixel mean intensity.
+#' 
+#' @return RGB raster representing the first 3 principal components
+#' 
+#' @importFrom grDevices rgb
+#' @import irlba
+#' @export
+#' 
+#' @aliases PCAImage
+#' 
+setMethod(
+  f = "PCAImage",
+  signature = signature(object = "msi.dataset"),
+  definition = function(object, alignToSample = TRUE) {
+    set.seed(123)
+    pca <- prcomp_irlba(object@matrix, center = TRUE, scale. = TRUE, n = 3,)
+    if (alignToSample) {
+      if (cor(apply(object@matrix, 1, mean), pca$x[, 1]) < 0) {
+        pca$x <- -pca$x
+      }
+    }
+    colors <- apply(pca$x, 2, function(x) (x - min(x)) / (max(x) - min(x)))
+    colors <- rgb(colors[, 1], colors[, 2], colors[, 3])
+    colors <- matrix(colors, object@nrow, object@ncol)
+    return (msImage(values = colors, name = 'PCA', scale = FALSE))
+  }
+)
+
+## totalIonCountMSI ----
+
+#' Generates an msImage representing pixels total-ion-counts. This
+#' image can be used to qualitatively evaluate the spatial heterogeneity of the
+#' sample.
+#' 
+#' @param object \link{msi.dataset-class} object.
+#' 
+#' @return \link{ms.image-class} object representing the total ion counts.
+#' 
+#' @export
+#' @aliases totalIonCountMSI
+#' 
+setMethod(
+  f = "totalIonCountMSI",
+  signature = signature(object = "msi.dataset"),
+  definition = function(object) {
+    tic <- apply(object@matrix, 1, function(x) sum(x, na.rm = TRUE))
+    im <- msImage(values = matrix(tic, object@nrow, object@ncol),
+                  name = "Total-ion-count", scale = FALSE)
+    return(im)
+  }
+)
+
+
+## numDetectedMSI ----
+
+#' Generates an msImage representing the number of detected peaks per pixel. This
+#' image can be used to qualitatively evaluate the spatial heterogeneity of the
+#' sample.
+#' 
+#' @param object \link{msi.dataset-class} object.
+#' 
+#' @return \link{ms.image-class} object representing the detected ions per pixel.
+#' 
+#' @export
+#' @aliases numDetectedMSI
+#' 
+setMethod(
+  f = "numDetectedMSI",
+  signature = signature(object = "msi.dataset"),
+  definition = function(object) {
+    ndet <- apply(object@matrix, 1, function(x) sum(x != 0, na.rm = TRUE))
+    im <- msImage(values = matrix(ndet, object@nrow, object@ncol),
+                  name = "Num. detected ions", scale = FALSE)
+    return(im)
+  }
+)
+
+
 ## getMZ ----
 
 #' Return the m/z vector.
@@ -92,12 +191,16 @@ setMethod(
 #' on first 10 principal components of peaks intensities.
 #'
 #' @param object \link{msi.dataset-class} object
+#' @param npcs int (default = 10). Number of principal components to calculate.
+#' @param ref string (default = "detected). Sample reference image used to align
+#' the clusters.
+#' @param invert boolean (default = FALSE). If FALSE, the clusters are inversely aligned to the
+#' sample reference image.
 #'
 #' @return \link{ms.image-class} object representing the binary mask image.
 #'
 #' @example R/examples/msiDataset_binKmeans.R
 #'
-#' @export
 #' @importFrom stats kmeans cor
 #' @importFrom irlba prcomp_irlba
 #' @aliases binKmeans
@@ -105,22 +208,40 @@ setMethod(
 setMethod(
   f = "binKmeans",
   signature = signature(object = "msi.dataset"),
-  definition = function(object) {
+  definition = function(object, ref = "detected", invert = FALSE,
+                        npcs = 10) {
     object@matrix[is.na(object@matrix)] <- 0
-    mean.sig <- apply(object@matrix, 1, mean)
-    n.comps <- min(dim(object@matrix) - 1, 10)
-    pca <- prcomp_irlba(object@matrix, n=n.comps)
-    if (cor(pca$x[, 1], mean.sig) < 0) {
-      pca$x <- (-1) * pca$x
-    }
+    npcs <- min(dim(object@matrix) - 1, 10)
+    pca <- prcomp_irlba(object@matrix, n = npcs)
+    
     y.clust <- kmeans(pca$x, centers = 2, iter.max = 1000, nstart = 5)
     y.clust <- (y.clust$cluster == 2) * 1
     
     values <- matrix(y.clust, object@nrow, object@ncol)
     bw.img <- msImage(values = values, name = "ROI", scale = FALSE)
-    if (cor(y.clust, mean.sig) < 0) {
-      bw.img <- invertImage(bw.img)
+    
+    if (ref == "detected") {
+      if (invert) {
+        if (cor(y.clust, c(object@numdetected@values)) > 0) {
+          bw.img <- invertImage(bw.img)
+        }
+      } else {
+        if (cor(y.clust, c(object@numdetected@values)) < 0) {
+          bw.img <- invertImage(bw.img)
+        }
+      }
+    } else if (ref == "tic") {
+      if (invert) {
+        if (cor(y.clust, c(object@totalioncount@values)) > 0) {
+          bw.img <- invertImage(bw.img)
+        }
+      } else {
+        if (cor(y.clust, c(object@totalioncount@values)) < 0) {
+          bw.img <- invertImage(bw.img)
+        }
+      }
     }
+    
     return(bw.img)
   }
 )
@@ -155,7 +276,6 @@ setMethod(
 #'
 #' @author Paolo Inglese \email{p.inglese14@imperial.ac.uk}
 #'
-#' @export
 #' @importFrom stats kmeans
 #' @import imager parallel
 #' @aliases binKmeans2
@@ -304,7 +424,6 @@ setMethod(
 #'
 #' @author Paolo Inglese \email{p.inglese14@imperial.ac.uk}
 #'
-#' @export
 #' @import imager e1071
 #' @importFrom stats predict quantile
 #' @aliases binSupervised
